MODIFIED from mbed official WiflyInterface (interface for Roving Networks Wifly modules). Numerous performance and reliability improvements (see the detailed documentation). Also, tracking changes in mbed official version to retain functional parity.

Dependents:   Smart-WiFly-WebServer PUB_WiflyInterface_Demo

Fork of WiflyInterface by mbed official

Resources

Derivative from mbed Official

  • Documentation update, improved consistency, documented parameters that were inadvertently omitted.
  • Avoid c++ string handling, which causes dynamic allocation and free, side effect, fewer CPU cycles spent for same purpose.
  • Fixed socket APIs to support non-blocking mode.
  • Increase communication baud-rate to Wifly module
  • sendCommand - added retries for improved robustness.
  • setConnectionState - method to force the connection state (used by TCPSocketServer)
  • gethostbyname - added a length parameter to the size of the buffer being written
  • flushIn - a private method to flush the input buffer
  • Changed the timeout from 500 to 2500 msec for commands - measured some at 700 to 850 msec.
  • Performance improvements - reduced some unnecessary delays.
  • Added additional security options for the wi-fi connection (that are supported by the WiFly module).
  • Added setSecurity API which permits revising the security when connecting to, or selecting from, one of several access points.
  • Improved DEBUG interface (slightly more consistent printout).
  • gathers information from the Wifly module on reboot (SW version info), which permits customizing behavior based on Wifly capabilities (like the improved security).
  • Avoid potential for recursive crash (if exit fails, it calls sendcommand, which calls exit...)
  • Update to support permissible SSID and PassCode lengths.

Robustness testing

I've had some mixed behavior with the Wifly module, some of which seems to be traceable to the module itself, and some in my derivative code. The result, after running for minutes, hours, sometimes days, it hangs and I have to reset the module.

To test, I created a fairly simple test program -

  • check for Watchdog induced reset and count it.
  • initialize the Watchdog for 60 sec timeout.
  • Init the Wifly interface and connect to my network.
  • Wait 10 seconds and force mbed_reset().

If the Watchdog induces the restart, then it is pretty clear that either:

  • The communications hung with the Wifly module causing the failure.
  • The Wifly module decided to go unresponsive.

If it gets to the end, it typically takes about 4 to 6 seconds for the boot and connect, then the 10 second delay.

But I can't really pin down the root cause easily. My strongest theory is that the Wifly module has rebooted, and since I don't store the high baud rate I configure it for, it resets back to 9600.

Also, one of the objectives for my revised send( ) is to avoid the c++ string, as that can fragment memory, and it wasn't very well bounded in behavior.

Latest tests:

Warm BootsWatchdog EventsNotes
100's30An early version of my derivative WiflyInterface, including my derivative of "send( )" API. Let's call this version 0.1.
26684My derivative WiflyInterface, but with the mbed official "send( )" API. Much improved. This was over the course of about 12 hours.
24003Most recent derivative - incremental change to "send( )", but this relative number does not rule out the Wifly module itself.

I think with these numbers, +/- 1 means that the changes have had no measurable effect. Which is good, since this incremental change eliminates the c++ string handling.

Test Software

This is pieces of a test program, clipped and copied to here. What I have compiled and run for hours and hours is almost exactly what you see. This uses this simple Watchdog library.

#include "mbed.h"
#include "WiflyInterface.h"
#include "Watchdog.h"

Serial pc(USBTX, USBRX);

Watchdog wd;
extern "C" void mbed_reset();

// Pinout for SmartBoard
WiflyInterface wifly(p9, p10, p30, p29, "ssid", "pass", WPA);

int main() {
    pc.baud(460800);                         // I like a snappy terminal
    
    wd.Configure(60.0);                     // Set time limit for the test to 1 minute
    LPC_RTC->GPREG0++;                      // Count boots here
    if (wd.WatchdogCausedReset()) {
        LPC_RTC->GPREG1++;                  // Count Watchdog events here
        pc.printf("\r\n\r\nWatchdog event.\r\n");
    }
    pc.printf("\r\nWifly Test: %d boots, %d watchdogs. %s %s\r\n", LPC_RTC->GPREG0, LPC_RTC->GPREG1, __DATE__, __TIME__);
    
    wifly.init(); // use DHCP
    pc.printf("Connect...  ");
    while (!wifly.connect());               // join the network
    pc.printf("Address is %s.  ", wifly.getIPAddress());
    pc.printf("Disconnect...  ");
    wifly.disconnect();
    pc.printf("OK. Reset in 10 sec...\r\n");
    wait(10);
    if (pc.readable()) {
        if (pc.getc() == 'r') {             // secret 'r'eset of the counters
            LPC_RTC->GPREG0 = 0;
            LPC_RTC->GPREG1 = 0;
            pc.printf("counters reset\r\n");
        }
    }
    mbed_reset();                           // reset here indicates successful communication
}
Revision:
10:a594fe035b36
Parent:
9:86105ba18d96
Child:
12:6270ced02019
--- a/Wifly/Wifly.h	Tue Jun 25 18:04:21 2013 +0000
+++ b/Wifly/Wifly.h	Thu Jun 27 17:12:35 2013 +0000
@@ -51,7 +51,8 @@
     WPA =       3,  // maintained for backward compatibility
     WPA2_PSK =  4,
     ADHOC =     6,
-    WPE_64 =    8
+    WPE_64 =    8,
+    WEP_64 =    8   // probably what the last one should have been
 };
 
 enum Protocol {
@@ -61,10 +62,10 @@
 
 /** the Wifly object
 *
+* This object controls the Wifly module.
 */
 class Wifly
 {
-
 public:
     /**
     * Constructor
@@ -75,20 +76,26 @@
     * @param tcp_status connection status pin of the wifi module (GPIO 6)
     * @param ssid ssid of the network
     * @param phrase WEP or WPA key
     * @param sec Security type (NONE, WEP_128 or WPA)
     */
     Wifly(PinName tx, PinName rx, PinName reset, PinName tcp_status, const char * ssid, const char * phrase, Security sec);
 
     /**
-    * Set the baud rate between the ARM and the WiFly module by first
-    * setting the WiFly module and then shifting the ARM to that speed.
+    * Set the baud rate between the ARM and the WiFly.
+    *
+    * This will try to set the WiFly module baud rate first and then
+    * set the ARM to match it. If it cannot get the proper acknowledge
+    * response, it will go on a hunt through the range of standard
+    * baud rates.
     *
     * @note Baud rate of 230400 has been tested without using flow
     *       control.
     *
     * @param baudrate is the desired baudrate.
+    * @returns true if it succeeded, which means that communications can continue, or false
+    *       if it failed to establish a communication link.
     */
-    void baud(int baudrate);
+    bool baud(int baudrate);
 
     /**
     * Connect the wifi module to the ssid contained in the constructor.
@@ -172,7 +179,6 @@
     */
     int putc(char c);
 
-
     /**
     * To enter in command mode (we can configure the module)
     *
@@ -215,7 +221,7 @@
     * @param resSize is the size of the field that res references, to avoid buffer overrun. If res is != NULL resSize is evaluated.
     * @param timeout is the time in msec to wait for the acknowledge
     *
-    * @return true if ACK has been found in the response from the wifi module. False otherwise or if there is no response in 5s.
+    * @return >=0 if the message was sent, -1 if there is no response within the timeout.
     */
     int send(const char * str, int len, const char * ACK = NULL, char * res = NULL, int resSize = 0, int timeout = DEFAULT_WAIT_RESP_TIMEOUT);
 
@@ -233,9 +239,9 @@
     bool sendCommand(const char * cmd, const char * ack = NULL, char * res = NULL, int resSize = 0, int timeout = DEFAULT_WAIT_RESP_TIMEOUT);
     
     /**
-    * Return true if the module is using dhcp
+    * Test if the current configuration is using dhcp
     *
-    * @returns true if the module is using dhcp
+    * @returns true if the module is using dhcp, false otherwise
     */
     bool isDHCP() {
         return state.dhcp;
@@ -265,8 +271,17 @@
     *
     * @returns the version information as a string, or NULL
     */
-    char * getWiflyVerString();
-
+    char * getWiflyVersionString();
+    
+    /**
+    * Get the software version from the Wifly module.
+    *
+    * This extracts the basic version number (e.g. 2.38, 4.00)
+    * as a float.
+    *
+    * @returns the software version number as a float.
+    */
+    float getWiflyVersion();
 
     static Wifly * getInstance() {
         return inst;
@@ -279,11 +294,12 @@
     int baudrate;       //DS for "instantaneous" shifts in baudrate
     char * phrase;
     char * ssid;
-    char * wiflyVer;
+    char * wiflyVersionString;
+    float swVersion;
     const char * ip;
     const char * netmask;
     const char * gateway;
-    int channel;
+    //int channel;
     CircBuffer<char> buf_wifly;
 
     static Wifly * inst;
@@ -302,7 +318,9 @@
     } State;
 
     State state;
+    void GatherLogonInfo();
     char * getStringSecurity();
+    void SetPhrase(char ** dst, const char * src);
 };
 
 #endif
@@ -75,20 +76,26 @@
     * @param ssid ssid of the network
     * @param phrase WEP or WPA key
-    * @param sec Security type (NONE, WEP_128 or WPA)
+    * @param sec Security type (NONE, WEP_128, WPA1, WPA | WPA_MIXED, WPA2_PSK, WEP_64 )
     */
     Wifly(PinName tx, PinName rx, PinName reset, PinName tcp_status, const char * ssid, const char * phrase, Security sec);
\ No newline at end of file